Package: TTR
Type: Package
Title: Technical Trading Rules
Version: 0.24.4
Authors@R: c(
  person(given="Joshua", family="Ulrich", role=c("cre","aut"), email="josh.m.ulrich@gmail.com"),
  person(given=c("Ethan","B."), family="Smith", role="ctb")
  )
Imports: xts (>= 0.10-0), zoo, curl
LinkingTo: xts
Enhances: quantmod
Suggests: RUnit
Description: A collection of over 50 technical indicators for creating technical trading rules. The package also provides fast implementations of common rolling-window functions, and several volatility calculations.
License: GPL (>= 2)
URL: https://github.com/joshuaulrich/TTR
BugReports: https://github.com/joshuaulrich/TTR/issues
NeedsCompilation: yes
Packaged: 2023-11-28 00:12:35 UTC; josh
Author: Joshua Ulrich [cre, aut],
  Ethan B. Smith [ctb]
Maintainer: Joshua Ulrich <josh.m.ulrich@gmail.com>
Repository: CRAN
Date/Publication: 2023-11-28 05:20:21 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:21:33 UTC; unix
Archs: TTR.so.dSYM
